#ifndef _TrackToFTSStore_H_
#define _TrackToFTSStore_H_

#include <rave/Track.h>
#include "TrackingTools/TrajectoryState/interface/FreeTrajectoryState.h"
#include <map>

class TrackToFTSStore
{
  /** 
   *  This is a singleton container that is used 
   *  to store RaveTrack to FreeTrajectoryState associations,
   *  such that we can retrieve the original track id
   *  when e.g. propagating a track
   */
  public:
    static TrackToFTSStore & Instance();
    void clear();
    void associate ( const rave::Track & t, 
                     const FreeTrajectoryState & fts );

    int getId ( const FreeTrajectoryState & fts ) const;
    bool has ( const FreeTrajectoryState & fts ) const;

    const FreeTrajectoryState & getFts ( const rave::Track & t ) const;

  private:
    TrackToFTSStore() {};
    TrackToFTSStore( const TrackToFTSStore & ) {};
    mutable std::map < int, FreeTrajectoryState > association_;

    struct FTSC {
      bool operator() ( const FreeTrajectoryState & f1, 
          const FreeTrajectoryState & f2 ) const
      {
        if ( (f1.momentum().mag() - f2.momentum().mag()) < -1e-5 ) return true;
        if ( (f1.position().mag() - f2.position().mag()) < -1e-5 ) return true;
        return false;
      }
    };
    mutable std::map < FreeTrajectoryState, int, FTSC > bassociation_;
};

#endif
